import os
import ctypes
import sys
import Record
import copy
import warnings


def initialize_file_list(prefix, start_file_idx, last_file_idx, ext, folder, zero_padding, max_characters_in_file):
    file_count = 0
    number_of_files = last_file_idx - start_file_idx + 1
    file_names = ((
                              ctypes.c_char * max_characters_in_file) * number_of_files)()  # setting up pointer to the char array which will hold the file names
    for i in range(start_file_idx, last_file_idx + 1):
        file_name_string = os.path.join(folder, prefix + str(i).rjust(zero_padding, '0') + ext)
        file_names[file_count] = ctypes.create_string_buffer(file_name_string.encode('utf-8'), max_characters_in_file)
        file_count = file_count + 1
    return file_names


def read_events(EventFileReaderDll, file_name):
    # Create buffers to be used by the DLL. Do not modify this section
    stringLength = 512  # strings will be a maximum of 512 characters.
    numberOfFieldsInRecord = 6  # each record has 6 fields.
    errorString = ctypes.create_string_buffer(stringLength)
    records = []
    error = EventFileReaderDll.LoadFiles(file_name, 1)
    if error == 0:  # file loaded successfully
        print("Files were loaded successfully\n")
    else:  # if LoadFile returned an error - if error code is zero, the function was successful
        EventFileReaderDll.GetErrorAsString(error,
                                            ctypes.addressof(errorString))  # return a string containing error message
        print(errorString.value.decode('utf-8'))  # print the error message
        sys.exit()

    numberOfRecordsInFile = ctypes.c_int()
    EventFileReaderDll.GetNumberOfRecords(ctypes.addressof(numberOfRecordsInFile))

    if error != 0:
        EventFileReaderDll.GetErrorAsString(error, ctypes.addressof(
            errorString))  # input pointer to string which will store error message
        print(errorString.value.decode('utf-8'))  # print the error message
        sys.exit()

    print("There are %d records in this file" % int(numberOfRecordsInFile.value))
    for i in range(0, numberOfRecordsInFile.value):
        myRecord = ((
                                ctypes.c_char * stringLength) * numberOfFieldsInRecord)()  # setting up pointer to the string array which will hold the record information
        error = EventFileReaderDll.GetIndexedRecord(i, myRecord)  # retrieve record

        if error != 0:  # if failed to retrieve record
            EventFileReaderDll.GetErrorAsString(error, ctypes.addressof(
                errorString))  # return a string containing error message
            print(errorString.value)  # print the error message
            sys.exit()
        else:
            event_number = float(myRecord[0].value)
            event_type = myRecord[1].value.decode('utf-8')
            timestamp = myRecord[2].value.decode('utf-8')
            time_ms_from_midnight = myRecord[3].value.decode('utf-8')
            time_source = myRecord[4].value.decode('utf-8')
            details = myRecord[5].value.decode('utf-8')
            records.append(
                Record.EventRecord(event_number, event_type, timestamp, time_ms_from_midnight, time_source, details))
    return records


def sort_events(event_records, data_event_records):
    start_recording_count = 0
    ms_in_day = 86400000
    recording_start_timestamps = []
    start_recording_indices = []
    stopped_recording_timestamps = []

    #find start recordings
    for event_idx in range(0, len(event_records)):
        if (event_records[event_idx].details == 'Started recording') & (event_records[event_idx].event_type == 'Mode change'):
            start_recording_count = start_recording_count + 1
            recording_start_timestamps.append(event_records[event_idx].time_ms_from_midnight)
            start_recording_indices.append(event_idx)
            if event_idx < len(event_records) - 1:  #this is not last event
                stopped_recording_timestamps.append(event_records[event_idx + 1].time_ms_from_midnight)
            else:
                stopped_recording_timestamps.append(ms_in_day) #if start recording is last event, then stop recording is missing so add all remaining files to this recording

    #find which files belong to which recordings
    recording_file_indices = [[], []]
    data_file_start_recording_timestamps = []
    first_file_in_recording = 0
    for recording_idx in range(0, len(recording_start_timestamps)):
        recording_crosses_midnight = False
        recording_file_indices.append([])
        for file_idx in range(first_file_in_recording, len(data_event_records)):
            if data_event_records[file_idx][0].time_ms_from_midnight == recording_start_timestamps[recording_idx]: # first file in a recording
                recording_file_indices[recording_idx].append(file_idx)
                data_file_start_recording_timestamps.append(data_event_records[file_idx][0].time_ms_from_midnight)
            else:
                if file_idx > 0:
                    if (data_event_records[file_idx][0].time_ms_from_midnight < data_event_records[file_idx-1][0].time_ms_from_midnight) & (data_event_records[file_idx][0].event_type != 'Event logging error'): #means this recording crosses midnight
                        recording_crosses_midnight = True
                temp_file_start_timestamp = float(data_event_records[file_idx][0].time_ms_from_midnight)
                temp_recording_stopped_timestamp = float(stopped_recording_timestamps[recording_idx])
                if recording_crosses_midnight:
                    temp_file_start_timestamp = temp_file_start_timestamp + ms_in_day
                    temp_recording_stopped_timestamp = temp_recording_stopped_timestamp + ms_in_day
                if temp_file_start_timestamp < temp_recording_stopped_timestamp:
                    recording_file_indices[recording_idx].append(file_idx)
                    first_file_in_recording = file_idx + 1



    # create a new list containing the events in order
    events_list_ordered = []
    event_idx = 0

    # remove all start and recording events from data files to remove doubles (these events appear in event files as well)
    for file in data_event_records:
        for event in file:
            if (event.details == 'Started recording') & (event.event_type == 'Mode change'):
                file.remove(event)

    for recording_idx in range(0, len(recording_start_timestamps)):
        events_list_ordered.extend(copy.deepcopy(event_records[event_idx:start_recording_indices[recording_idx]+1])) # add events from event file until and including start recording
        event_idx = start_recording_indices[recording_idx]+2 #move to next chunk of events
        for file_idx_in_recording in recording_file_indices[recording_idx]: #add events from files
            events_list_ordered.extend(copy.deepcopy(data_event_records[file_idx_in_recording]))
    if event_idx < len(event_records)-1:
        events_list_ordered.extend(copy.deepcopy(event_records[event_idx:len(event_records)]))
    return events_list_ordered







