# This program is an example script that demonstrates how to use the event file reader dll. It comes with 2 dlls: the
# dll that reads the data (Event_File_Reader_9_0.dll) and the wrapper dll (Event_File_Reader_Dll_UM.dll).
# This example demonstrates how to load an event log file, display the header and the number of records in the file,
# retrieve a record from the file, and check for errors.
#Tested in Python 3.6, not tested for case where a recording crosses midnight

# N.B. Event_File_Reader_9_0.dll must be in the same folder as your python.exe


import ctypes
import math
import os
import warnings

import event_reader_python_example_functions as er_fun
from itertools import chain


# User settings

wrapperDll = ''  # enter the full or relative path of the wrapper dll
dataFolder = '' #enter the full path of your data folder
# Load wrapper DLL
EventFileReaderDll = ctypes.cdll.LoadLibrary(wrapperDll)
file_collection_type = "Block" #"Flat" or "Block" - depends on your file format

max_characters_in_file = 260  # constant, do not change this value
# for block file format
if file_collection_type == "Block":
    include_event_file = True  # if to include EVENT000 or not
    data_file_prefix = "NEUR" # 4 letter prefix of your data file names
    event_file_prefix = "EVENT" #5 letter prefix of your event file name
    data_digit_padding = 4  # data file naming convention is AAAAxxxx.DF1 where AAAA is the prefix, xxxx is the 4 digit index of the file
    event_digit_padding = 3  # event file naming convention is EVENTxxx.DF1 where xxx is the 3 digit index of the file
    extension = ".DF1"
    start_file_index = 0
    last_file_index = 33
    event_start_file_index = 0
    event_last_file_index = 0
    number_of_files = last_file_index - start_file_index + 1
    number_of_event_files = event_last_file_index - event_start_file_index + 1

    file_names = er_fun.initialize_file_list(data_file_prefix, start_file_index, last_file_index, extension, dataFolder,
                                             data_digit_padding, max_characters_in_file)
    event_records = []
    data_event_records = []

    # get events from special event file
    if include_event_file:
        event_file_names = er_fun.initialize_file_list(event_file_prefix, event_start_file_index, event_last_file_index,
                                                       extension, dataFolder, event_digit_padding,
                                                       max_characters_in_file)
        for event_file_idx in range(0, number_of_event_files):
            event_records.append(er_fun.read_events(EventFileReaderDll, event_file_names[event_file_idx]))
        event_records = list(chain.from_iterable(event_records))  # flatten event files into 1d list

    # get events from data files
    for file_idx in range(0, number_of_files):
        data_event_records.append(er_fun.read_events(EventFileReaderDll, file_names[file_idx]))

elif file_collection_type == "Flat":
    number_of_event_files = 1  # flat file format has only 1 event log file
    file_names = ((ctypes.c_char * max_characters_in_file) * number_of_event_files)()  # setting up pointer to the string array which will hold the record information
    file_names[0] = ctypes.create_string_buffer(os.path.join(dataFolder, "EVENTLOG.NLE").encode('utf-8'),
                                                max_characters_in_file)
    include_event_file = False
    data_file_prefix = "EVENTLOG"
    event_digit_padding = 0
    start_file_index = 0
    last_file_index = 0
    data_digit_padding = 0
    extension = ".NLE"
    data_event_records = []
    data_event_records.append(er_fun.read_events(EventFileReaderDll, file_names[0]))
    # end of flat file format







# if includes event file, must re-order events chronologically
if include_event_file:
    sorted_events = er_fun.sort_events(event_records, data_event_records)
else:
    sorted_events = list(chain.from_iterable(data_event_records)) #flatten records



for event_idx in range(1, len(sorted_events)):
    if (sorted_events[event_idx].event_type =='Continuation record'): #change continuation record to continued
        sorted_events[event_idx].event_type = '...Continued'
    if (sorted_events[event_idx].event_type == '...Continued'): #fix numbering
        sorted_events[event_idx].event_number = round(sorted_events[event_idx-1].event_number + 0.1, 1)
    else:
        sorted_events[event_idx].event_number = math.floor(sorted_events[event_idx-1].event_number) + 1

for i in range(0, len(sorted_events)):
    print("Event number: %s\nEvent type: %s\nTimestamp: %s\nTime (ms from midnight): %s\nTime source: %s\nDetails: %s\n"
          % (sorted_events[i].event_number, sorted_events[i].event_type, sorted_events[i].timestamp, sorted_events[i].time_ms_from_midnight,
             sorted_events[i].time_source, sorted_events[i].details)) # print values in records



del wrapperDll



